@@ -1,6 +1,6 @@
 package Text::CSV_XS;
 
-# Copyright (c) 2007-2014 H.Merijn Brand.  All rights reserved.
+# Copyright (c) 2007-2015 H.Merijn Brand.  All rights reserved.
 # Copyright (c) 1998-2001 Jochen Wiedmann. All rights reserved.
 # Copyright (c) 1997 Alan Citterman.       All rights reserved.
 #
@@ -9,14 +9,12 @@ package Text::CSV_XS;
 
 # HISTORY
 #
-# Written by:
-#    Jochen Wiedmann <joe@ispsoft.de>
-#
-# Based on Text::CSV by:
-#    Alan Citterman <alan@mfgrtl.com>
-#
-# Extended and Remodelled by:
+# 0.24 -
 #    H.Merijn Brand (h.m.brand@xs4all.nl)
+# 0.10 - 0.23
+#    Jochen Wiedmann <joe@ispsoft.de>
+# Based on (the original) Text::CSV by:
+#    Alan Citterman <alan@mfgrtl.com>
 
 require 5.006001;
 
@@ -28,7 +26,7 @@ use DynaLoader ();
 use Carp;
 
 use vars   qw( $VERSION @ISA @EXPORT_OK );
-$VERSION   = "1.12";
+$VERSION   = "1.14";
 @ISA       = qw( DynaLoader Exporter );
 @EXPORT_OK = qw( csv );
 bootstrap Text::CSV_XS $VERSION;
@@ -37,6 +35,11 @@ sub PV { 0 }
 sub IV { 1 }
 sub NV { 2 }
 
+if ($] < 5.008002) {
+    no warnings "redefine";
+    *utf8::decode = sub {};
+    }
+
 # version
 #
 #   class/object method expecting no arguments and returning the version
@@ -114,6 +117,7 @@ sub _check_sanity
 {
     my $self = shift;
 
+    my $eol = $self->{eol};
     my $sep = $self->{sep};
     defined $sep && length ($sep) or $sep = $self->{sep_char};
     my $quo = $self->{quote};
@@ -123,11 +127,22 @@ sub _check_sanity
 #    use DP;::diag ("SEP: '", DPeek ($sep),
 #	        "', QUO: '", DPeek ($quo),
 #	        "', ESC: '", DPeek ($esc),"'");
-    # sep_char cannot be undefined
-    defined $quo && $quo eq $sep  and return 1001;
-    defined $esc && $esc eq $sep  and return 1001;
-
-    defined $_ && $_ =~ m/[\r\n]/ and return 1003 for $sep, $quo, $esc;
+    if (defined $sep) {	# sep_char cannot be undefined
+	length ($sep) > 16	and return 1006;
+	$sep =~ m/[\r\n]/	and return 1003;
+	}
+    if (defined $quo) {
+	$quo eq $sep		and return 1001;
+	length ($quo) > 16	and return 1007;
+	$quo =~ m/[\r\n]/	and return 1003;
+	}
+    if (defined $esc) {
+	$esc eq $sep		and return 1001;
+	$esc =~ m/[\r\n]/	and return 1003;
+	}
+    if (defined $eol) {
+	length ($eol) > 16	and return 1005;
+	}
 
     return _unhealthy_whitespace ($self, $self->{allow_whitespace});
     } # _check_sanity
@@ -160,8 +175,7 @@ sub new
     for (keys %attr) {
 	if (m/^[a-z]/ && exists $def_attr{$_}) {
 	    # uncoverable condition false
-	    defined $attr{$_} && $] >= 5.008002 && m/_char$/ and
-		utf8::decode ($attr{$_});
+	    defined $attr{$_} && m/_char$/ and utf8::decode ($attr{$_});
 	    next;
 	    }
 #	croak?
@@ -241,7 +255,7 @@ sub _set_attr_C
 {
     my ($self, $name, $val, $ec) = @_;
     defined $val or $val = 0;
-    $] >= 5.008002 and utf8::decode ($val);
+    utf8::decode ($val);
     $self->{$name} = $val;
     $ec = _check_sanity ($self) and
 	croak ($self->SetDiag ($ec));
@@ -283,9 +297,10 @@ sub quote
     if (@_) {
 	my $quote = shift;
 	defined $quote or $quote = "";
-	$] >= 5.008002 and utf8::decode ($quote);
+	utf8::decode ($quote);
 	my @b = unpack "U0C*", $quote;
 	if (@b > 1) {
+	    @b > 16 and croak ($self->SetDiag (1007));
 	    $self->quote_char ("\0");
 	    }
 	else {
@@ -326,9 +341,10 @@ sub sep
     if (@_) {
 	my $sep = shift;
 	defined $sep or $sep = "";
-	$] >= 5.008002 and utf8::decode ($sep);
+	utf8::decode ($sep);
 	my @b = unpack "U0C*", $sep;
 	if (@b > 1) {
+	    @b > 16 and croak ($self->SetDiag (1006));
 	    $self->sep_char ("\0");
 	    }
 	else {
@@ -352,6 +368,7 @@ sub eol
     if (@_) {
 	my $eol = shift;
 	defined $eol or $eol = "";
+	length ($eol) > 16 and croak ($self->SetDiag (1005));
 	$self->{eol} = $eol;
 	$self->_cache_set ($_cache_id{eol}, $eol);
 	}
@@ -578,32 +595,34 @@ sub error_diag
 	if ($diag[0] && $diag[0] != 2012) {
 	    my $msg = "# CSV_XS ERROR: $diag[0] - $diag[1] \@ rec $diag[3] pos $diag[2]\n";
 	    $diag[4] and $msg =~ s/$/ field $diag[4]/;
-	    if ($self && ref $self) {	# auto_diag
-		if ($self->{diag_verbose} and $self->{_ERROR_INPUT}) {
-		    $msg .= "$self->{_ERROR_INPUT}'\n";
-		    $msg .= " " x ($diag[2] - 1);
-		    $msg .= "^\n";
-		    }
 
-		my $lvl = $self->{auto_diag};
-		if ($lvl < 2) {
-		    my @c = caller (2);
-		    if (@c >= 11 && $c[10] && ref $c[10] eq "HASH") {
-			my $hints = $c[10];
-			(exists $hints->{autodie} && $hints->{autodie} or
-			 exists $hints->{"guard Fatal"} &&
-			!exists $hints->{"no Fatal"}) and
-			    $lvl++;
-			# Future releases of autodie will probably set $^H{autodie}
-			#  to "autodie @args", like "autodie :all" or "autodie open"
-			#  so we can/should check for "open" or "new"
-			}
-		    }
-		$lvl > 1 ? die $msg : warn $msg;
-		}
-	    else {	# called without args in void context
+	    unless ($self && ref $self) {	# auto_diag
+	    	# called without args in void context
 		warn $msg;
+		return;
 		}
+
+	    if ($self->{diag_verbose} and $self->{_ERROR_INPUT}) {
+		$msg .= "$self->{_ERROR_INPUT}'\n";
+		$msg .= " " x ($diag[2] - 1);
+		$msg .= "^\n";
+		}
+
+	    my $lvl = $self->{auto_diag};
+	    if ($lvl < 2) {
+		my @c = caller (2);
+		if (@c >= 11 && $c[10] && ref $c[10] eq "HASH") {
+		    my $hints = $c[10];
+		    (exists $hints->{autodie} && $hints->{autodie} or
+		     exists $hints->{"guard Fatal"} &&
+		    !exists $hints->{"no Fatal"}) and
+			$lvl++;
+		    # Future releases of autodie will probably set $^H{autodie}
+		    #  to "autodie @args", like "autodie :all" or "autodie open"
+		    #  so we can/should check for "open" or "new"
+		    }
+		}
+	    $lvl > 1 ? die $msg : warn $msg;
 	    }
 	return;
 	}
@@ -946,7 +965,7 @@ sub _csv_attr
 	}
     elsif (ref $in or "GLOB" eq ref \$in) {
 	if (!ref $in && $] < 5.008005) {
-	    $fh = \*$in;
+	    $fh = \*$in; # uncoverable statement ancient perl version required
 	    }
 	else {
 	    $fh = $in;
@@ -971,6 +990,9 @@ sub _csv_attr
     my $cboi = delete $attr{callbacks}{on_in}       ||
 	       delete $attr{on_in};
 
+    my $fltr = delete $attr{filter};
+    ref $fltr eq "HASH" or $fltr = undef;
+
     defined $attr{auto_diag} or $attr{auto_diag} = 1;
     my $csv = Text::CSV_XS->new (\%attr) or croak $last_new_err;
 
@@ -983,6 +1005,7 @@ sub _csv_attr
 	hdrs => $hdrs,
 	key  => $key,
 	frag => $frag,
+	fltr => $fltr,
 	cbai => $cbai,
 	cbbo => $cbbo,
 	cboi => $cboi,
@@ -1030,10 +1053,10 @@ sub csv
 		}
 	    }
 	else { # aoh
-	    my @hdrs = ref $hdrs ? @{$hdrs}
-				 : map { $hdr{$_} || $_ } keys %{$in->[0]};
+	    my @hdrs = ref $hdrs ? @{$hdrs} : keys %{$in->[0]};
 	    defined $hdrs or $hdrs = "auto";
-	    ref $hdrs || $hdrs eq "auto" and $csv->print ($fh, \@hdrs);
+	    ref $hdrs || $hdrs eq "auto" and
+		$csv->print ($fh, [ map { $hdr{$_} || $_ } @hdrs ]);
 	    for (@{$in}) {
 		$c->{cboi} and $c->{cboi}->($csv, $_);
 		$c->{cbbo} and $c->{cbbo}->($csv, $_);
@@ -1045,9 +1068,8 @@ sub csv
 	return 1;
 	}
 
-    ref $in eq "CODE" and croak "CODE only valid fro in when using out";
-
     my $key = $c->{key} and $hdrs ||= "auto";
+    $c->{fltr} && grep m/\D/ => keys %{$c->{fltr}} and $hdrs ||= "auto";
     if (defined $hdrs && !ref $hdrs) {
 	if ($hdrs eq "skip") {
 	    $csv->getline ($fh); # discard;
@@ -1058,6 +1080,24 @@ sub csv
 	    }
 	}
 
+    if ($c->{fltr}) {
+	my %f = %{$c->{fltr}};
+	# convert headers to index
+	if (ref $hdrs) {
+	    my @hdr = @{$hdrs};
+	    for (0 .. $#hdr) {
+		exists $f{$hdr[$_]} and $f{$_ + 1} = delete $f{$hdr[$_]};
+		}
+	    }
+	$csv->callbacks (after_parse => sub {
+	    my ($csv, $r) = @_;
+	    foreach my $fld (sort keys %f) {
+		local $_ = $r->[$fld - 1];
+		$f{$fld}->() or return \"skip";
+		}
+	    });
+	}
+
     my $frag = $c->{frag};
     my $ref = ref $hdrs
 	? # aoh
@@ -1073,11 +1113,13 @@ sub csv
     $ref or Text::CSV_XS->auto_diag;
     $c->{cls} and close $fh;
     if ($ref and $c->{cbai} || $c->{cboi}) {
-	for (@{$ref}) {
-	    $c->{cbai} and $c->{cbai}->($csv, $_);
-	    $c->{cboi} and $c->{cboi}->($csv, $_);
+	foreach my $r (@{$ref}) {
+	    $c->{cbai} and $c->{cbai}->($csv, $r);
+	    $c->{cboi} and $c->{cboi}->($csv, $r);
 	    }
 	}
+
+    defined wantarray or return csv (in => $ref, headers => $hdrs);
     return $ref;
     } # csv
 
@@ -1085,7 +1127,7 @@ sub csv
 
 __END__
 
-=encoding iso-8859-1
+=encoding utf-8
 
 =head1 NAME
 
@@ -1691,7 +1733,7 @@ record (unless quotation was added because of other reasons).
     quote_space    => 0,
     });
 
- my $row = $csv->parse (q{1,,"", ," ",f,"g","h""h",hlp,"hlp"});
+ my $row = $csv->parse (q{1,,"", ," ",f,"g","h""h",hëlp,"hélp"});
 
  $csv->print (*STDOUT, \@row);
  # 1,,, , ,f,g,"h""h",h?lp,h?lp
@@ -2368,7 +2410,7 @@ itself (e.g. C<*STDIN>), or a reference to a scalar (e.g. C<\q{1,2,"csv"}>).
 
 When used with L</out>, C<in> should be a reference to a CSV structure (AoA
 or AoH)  or a CODE-ref that returns an array-reference or a hash-reference.
-The code-ref will be invoked with no arguments and .
+The code-ref will be invoked with no arguments.
 
  my $aoa = csv (in => "file.csv");
 
@@ -2378,6 +2420,20 @@ The code-ref will be invoked with no arguments and .
  my $csv = [ [qw( Foo Bar )], [ 1, 2 ], [ 2, 3 ]];
  my $err = csv (in => $csv, out => "file.csv");
 
+If called in void context without the L</out> attribute, the resulting ref
+will be used as input to a subsequent call to csv:
+
+ csv (in => "file.csv", filter => { 2 => sub { length > 2 }})
+
+will be a shortcut to
+
+ csv (in => csv (in => "file.csv", filter => { 2 => sub { length > 2 }}))
+
+where, in the absence of the C<out> attribute, this is a shortcut to
+
+ csv (in  => csv (in => "file.csv", filter => { 2 => sub { length > 2 }}),
+      out => *STDOUT)
+
 =head3 out
 X<out>
 
@@ -2556,7 +2612,8 @@ This callback is invoked after parsing with  L</getline>  only if no  error
 occurred.  The callback is invoked with two arguments:   the current C<CSV>
 parser object and an array reference to the fields parsed.
 
-The return code of the callback is ignored.
+The return code of the callback is ignored  unless it is a reference to the
+string "skip", in which case the record will be skipped in L</getline_all>.
 
  sub add_from_db
  {
@@ -2568,6 +2625,34 @@ The return code of the callback is ignored.
  my $aoa = csv (in => "file.csv", callbacks => {
      after_parse => \&add_from_db });
 
+This hook can be used for validation:
+X<validation>
+
+=over 2
+
+=item FAIL
+
+Die if any of the records does not validate a rule:
+
+ after_parse => sub {
+     $_[1][4] =~ m/^[0-9]{4}\s?[A-Z]{2}$/ or
+         die "5th field does not have a valid Dutch zipcode";
+     }
+
+=item DEFAULT
+
+Replace invalid fields with a default value:
+
+ after_parse => sub { $_[1][2] =~ m/^\d+$/ or $_[1][2] = 0 }
+
+=item SKIP
+
+Skip records that have invalid fields (only applies to L</getline_all>):
+
+ after_parse => sub { $_[1][0] =~ m/^\d+$/ or return \"skip"; }
+
+=back
+
 =item before_print
 X<before_print>
 
@@ -2601,6 +2686,7 @@ but only feature the L</csv> function.
 
   csv (in        => "file.csv",
        callbacks => {
+           filter       => { 6 => sub { $_ > 15 } },    # first
            after_parse  => sub { say "AFTER PARSE";  }, # first
            after_in     => sub { say "AFTER IN";     }, # second
            on_in        => sub { say "ON IN";        }, # third
@@ -2618,6 +2704,28 @@ but only feature the L</csv> function.
 
 =over 2
 
+=item filter
+X<filter>
+
+This callback can be used to filter records.  It is called just after a new
+record has been scanned.  The callback accepts a hashref where the keys are
+the index to the row (the field number, 1-based) and the values are subs to
+return a true or false value.
+
+ csv (in => "file.csv", filter => {
+            3 => sub { m/a/ },       # third field should contain an "a"
+            5 => sub { length > 4 }, # length of the 5th field minimal 5
+            });
+
+If the keys to the filter contain any character that in not a digit it will
+also implicitly set  L</headers> to C<auto>  unless L</headers> was already
+passed as argument.  When headers are active, returning an array of hashes,
+the filter is not applicable to the header itself.
+
+ csv (in => "file.csv", filter => { foo => sub { $_ > 4 }});
+
+All sub results should match, as in AND.
+
 =item after_in
 X<after_in>
 
@@ -2749,6 +2857,12 @@ Rewrite C<CSV> files with C<;> as separator character to well-formed C<CSV>:
  use Text::CSV_XS qw( csv );
  csv (in => csv (in => "bad.csv", sep_char => ";"), out => *STDOUT);
 
+As C<STDOUT> is now default in L</csv>, a one-liner converting a UTF-16 CSV
+file with BOM and TAB-separation to valid UTF-8 CSV could be:
+
+ $ perl -C3 -MText::CSV_XS=csv -we\
+    'csv(in=>"utf16tab.csv",encoding=>"utf16",sep=>"\t")' >utf8.csv
+
 =head2 Dumping database tables to CSV
 
 Dumping a database table can be simple as this (TIMTOWTDI):
@@ -2778,8 +2892,8 @@ Dumping a database table can be simple as this (TIMTOWTDI):
 For more extended examples, see the F<examples/> C<1>) sub-directory in the
 original distribution or the git repository C<2>).
 
- 1. http://repo.or.cz/w/Text-CSV_XS.git?a=tree;f=examples
- 2. http://repo.or.cz/w/Text-CSV_XS.git
+ 1. https://github.com/Tux/Text-CSV_XS/tree/master/examples
+ 2. https://github.com/Tux/Text-CSV_XS
 
 The following files can be found there:
 
@@ -2928,10 +3042,6 @@ No guarantees, but this is what I had in mind some time ago:
 
 DIAGNOSTICS section in pod to *describe* the errors (see below)
 
-=item *
-
-Multi-byte quotation support
-
 =back
 
 =head1 EBCDIC
@@ -3042,6 +3152,24 @@ X<1004>
 The L<C<callbacks>|/Callbacks>  attribute only allows one to be C<undef> or
 a hash reference.
 
+=item *
+1005 "INI - EOL too long"
+X<1005>
+
+The value passed for EOL is exceeding its maximum length (16).
+
+=item *
+1006 "INI - SEP too long"
+X<1006>
+
+The value passed for SEP is exceeding its maximum length (16).
+
+=item *
+1007 "INI - QUOTE too long"
+X<1007>
+
+The value passed for QUOTE is exceeding its maximum length (16).
+
 =item *
 2010 "ECR - QUO char inside quotes followed by CR not part of EOL"
 X<2010>
@@ -3230,7 +3358,7 @@ L</csv> function. See ChangeLog releases 0.25 and on.
 
 =head1 COPYRIGHT AND LICENSE
 
- Copyright (C) 2007-2014 H.Merijn Brand.  All rights reserved.
+ Copyright (C) 2007-2015 H.Merijn Brand.  All rights reserved.
  Copyright (C) 1998-2001 Jochen Wiedmann. All rights reserved.
  Copyright (C) 1997      Alan Citterman.  All rights reserved.
 
